"""

Copyright (c) 2016-2020 Vanessa Sochat

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

from pokemon.master import get_pokemon, catch_em_all, get_trainer


def get_ascii(pid=None, name=None, pokemons=None, return_pokemons=False, message=None):
    """get_ascii will return ascii art for a pokemon based on a name or pid.
    :param pid: the pokemon ID to return
    :param name: the pokemon name to return
    :param return_pokemons: return catches (default False)
    :param message: add a message to the ascii
    """
    pokemon = get_pokemon(name=name, pid=pid, pokemons=pokemons)
    printme = message
    if len(pokemon) > 0:
        for pid, data in pokemon.items():
            if message == None:
                printme = data["name"].capitalize()
            print("%s\n\n%s" % (data["ascii"], printme))

    if return_pokemons == True:
        return pokemon


def get_avatar(name, pokemons=None, print_screen=True, include_name=True):
    """get_avatar will return a unique pokemon for a specific avatar based on the hash
    :param name: the name to look up
    :param print_screen: if True, will print ascii to the screen (default True) and not return
    :param include_name: if True, will add name (minus end of address after @) to avatar
    """
    if pokemons is None:
        pokemons = catch_em_all()

    # The IDs are numbers between 1 and the max
    number_pokemons = len(pokemons)

    trainer = get_trainer(name)

    pid = str(trainer % number_pokemons)
    pokemon = get_pokemon(pid=pid, pokemons=pokemons)

    avatar = pokemon[pid]["ascii"]
    if include_name is True:
        avatar = "%s\n\n%s" % (avatar, name.split("@")[0])
    if print_screen is True:
        print(avatar)
    return avatar



"""

Copyright (c) 2016-2020 Vanessa Sochat

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

from PIL import Image

"""Credit goes to https://www.hackerearth.com/notes/beautiful-python-a-simple-ascii-art-generator-from-images/
The one modification I added was to scale the new_width by 2, because text characters tend to be thinner than they
are wide, and the current method produced images that were (generally) too tall!
"""
ASCII_CHARS = ["#", "?", "%", ".", "S", "+", ".", "*", ":", ",", "@"]


def scale_image(image, new_width):
    """Resizes an image preserving the aspect ratio.
    """
    (original_width, original_height) = image.size
    aspect_ratio = original_height / float(original_width)
    new_height = int(aspect_ratio * new_width)
    # This scales it wider than tall, since characters are biased
    new_image = image.resize((new_width * 2, new_height))
    return new_image


def convert_to_grayscale(image):
    return image.convert("L")


def map_pixels_to_ascii_chars(image, range_width=25):
    """Maps each pixel to an ascii char based on the range
    in which it lies.
    0-255 is divided into 11 ranges of 25 pixels each.
    """
    pixels_in_image = list(image.getdata())
    pixels_to_chars = [
        ASCII_CHARS[pixel_value // range_width] for pixel_value in pixels_in_image
    ]
    return "".join(pixels_to_chars)


def convert_image_to_ascii(image, new_width):
    image = scale_image(image, new_width)
    image = convert_to_grayscale(image)

    pixels_to_chars = map_pixels_to_ascii_chars(image)
    len_pixels_to_chars = len(pixels_to_chars)

    image_ascii = [
        pixels_to_chars[index : index + new_width * 2]
        for index in range(0, len_pixels_to_chars, new_width * 2)
    ]
    return "\n".join(image_ascii)


def handle_image_conversion(image_filepath, new_width=100):
    image = None
    try:
        image = Image.open(image_filepath)
    except:
        print(
            "Unable to open image file {image_filepath}.".format(
                image_filepath=image_filepath
            )
        )
        print(e)
        return
    image_ascii = convert_image_to_ascii(image, new_width)
    print(image_ascii)
    return image_ascii


if __name__ == "__main__":
    import sys

    image_file_path = sys.argv[1]
    handle_image_conversion(image_file_path)



"""

Copyright (c) 2016-2020 Vanessa Sochat

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

from pokemon.utils import get_installdir, load_json
from random import choice
import hashlib
import sys

base = get_installdir()


def get_pokemon(pid=None, name=None, pokemons=None):
    """get_pokemon will return a pokemon with a specific ID, or if none is given,
    will select randomly. First the pid will be used, then the name, then any filters.
    :param pid: the pokemon ID to return
    :param pokemons: the pokemons data structure
    """
    if pokemons == None:
        pokemons = catch_em_all()

    # First see if we want to find a pokemon by name
    if name is not None:
        catches = lookup_pokemon(field="name", value=name, pokemons=pokemons)
        if catches is not None:
            return catches
        print("We don't have a pokemon called %s" % name)
        sys.exit(1)

    # Next see if they want a random pokemon
    if pid is None:
        choices = list(pokemons.keys())
        pid = int(choice(choices))

    # Retrieve the random, or user selected pokemon
    if pid is not None and str(pid) in pokemons.keys():
        return {pid: pokemons[str(pid)]}

    else:
        print("Cannot find pokemon with this criteria!")


def get_trainer(name):
    """return the unique id for a trainer, determined by the md5 sum
    """
    name = name.lower()
    return int(hashlib.md5(name.encode("utf-8")).hexdigest(), 16) % 10 ** 8


def catch_em_all(data_file=None, return_names=False):
    """catch_em_all returns the entire database of pokemon, a base function for starting
    :param data_file: location of pokemons.json data file (not required)
    """
    if data_file == None:
        data_file = "%s/database/pokemons.json" % (base)

    pokemons = load_json(data_file)

    if return_names is True:
        names = []
        for key, meta in pokemons.items():
            names.append(meta["name"])
        return names
    return pokemons


def lookup_pokemon(field, value, pokemons=None):
    """lookup_pokemon will search a particular field (name) for a value. If no pokemons
    data structure is provided, all will be used.
    :param field: the field to look up.
    :param pokemons: the pokemons data structure
    """
    if pokemons == None:
        pokemons = catch_em_all()

    catches = {}
    for pid, data in pokemons.items():
        if isinstance(data[field], list):
            for entry in data[field]:
                found = search_entry(entry, value)
                if found == True:
                    catches[pid] = data
        else:
            found = search_entry(data[field], value)
            if found == True:
                catches[pid] = data

    if len(catches) > 0:
        return catches
    return None


def search_entry(field, value):
    if isinstance(field, float) or isinstance(field, int):
        if field == value:
            return True
    elif isinstance(field, str) or isinstance(field, unicode):
        if field.lower() == value.lower():
            return True
    return False



from setuptools import setup, find_packages
import os

with open('README.md') as filey:
    LONG_DESCRIPTION = filey.read()

setup(
    # Application name:
    name="pokemon",

    # Version number:
    version="0.35",

    # Application author details:
    author="Vanessa Sochat",
    author_email="vsochat@stanford.edu",

    # Packages
    packages=find_packages(),

    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",

    # Data files
    include_package_data=True,
    zip_safe=False,

    # Details
    url="http://www.github.com/vsoch/pokemon",
    license="LICENSE",
    description="ascii database of pokemon... in python!",
    keywords="pokemon, avatar, ascii, gravatar",
    entry_points={"console_scripts": ["pokemon=pokemon.scripts:main",],},
)
